/******************************************************************************
 *  Filename:       i2c.c
 *  Date:           03/15/13
 *  File Verison:   1.1
 *
 *  Author:         Amira Ahsan
 *                  Prarthana Ranjit
 *  Team:           LFEV'13
 *
 * ****************************************************************************
 *  Architecture:   Enhanced Midrange PIC
 *  Processor:      PIC16LF1827
 *  Compiler:       MPLAB X XC8
 *
 *
 * ****************************************************************************
 *  Description:    Sets up i2c communication
 *
 *
 * ****************************************************************************
 *  Pin Assignments:
 *                  
 *
 *******************************************************************************/

#include <xc.h>
#include <math.h>
#include <stdint.h>


// configuring the bits
#pragma config MCLRE= OFF        //MCLR/VPP pin function is digital input
#pragma config FOSC = INTOSC     // Selects the internal oscillator
#pragma config WDTE = OFF        //Turns off the Watchdog Timer
#pragma config LVP = OFF         //Low-voltage programming disabled
#pragma config PLLEN= OFF

#define TOFFSET (256-250+3)      //timer0 offset such that it overflows
                                 //after 250 cycles - 125 microsecond


/*I2C transmission and receiving Buffer Allocation*/
unsigned char txByte[6];
unsigned char rxByte[4];                  // rxByte[0]=address, rxByte[1]=
unsigned char flush;
volatile int i =0;
volatile int it = 0;
volatile unsigned char i2c_complete =0;  //I2C command complete indicator


//the interupt service routine
void interrupt isr(void){

    static uint16_t cnt_t0=0;                    // to count number of overflows

   // LATAbits.LATA1= 1;
    //LATAbits.LATA1= 0;

    if(PIR1bits.SSP1IF){                        //If I2C interrupt flag is set

        PIR1bits.SSP1IF = 0;  //clear i2c interrupt flag
      //  LATAbits.LATA2=1;
        //LATAbits.LATA2=0;

        if(SSP1STATbits.R_nW == 1){          //If a read command
            i = 0;
            flush = SSP1BUF;                 //if read flush the buffer

          //  LATAbits.LATA3=1;
            //LATAbits.LATA3=0;

            if(rxByte[1] < 0x04){ //given its a valid command

             //send the requested stuff to master
                txByte[0] = rxByte[0];          //set transmission buffer
                txByte[1] = rxByte[1];          // the command number
                txByte[2] = rxByte[2];
                txByte[3] = rxByte[3];
             }
            if(SSP1CON1bits.CKP == 0){          //if i2c clock is being held low
               // LATAbits.LATA4=1;
                //LATAbits.LATA4=0;

                SSP1BUF = txByte[it];           //write to i2c buffer
                SSP1CON1bits.CKP = 1;           //release i2c clock line
                it++;
            }

          //interrupt following the transfer of the last byte
             if(SSP1CON2bits.ACKSTAT == 1){

                //LATAbits.LATA6=1;
                //LATAbits.LATA6=0;
                it=0;
            }

        }else{                                  //a write command

            //LATAbits.LATA7=1;
            //LATAbits.LATA7=0;
            rxByte[i] = SSP1BUF;                //read i2c buffer
            i++;
            //if a complete command has been received
            if((i)==4){
                //LATAbits.LATA3=1;
                //LATAbits.LATA3=0;
                i = 0;
                i2c_complete = 1;               //signifies that full i2c command was received
            }
        }
     }
}

void main(){

  _delay(1000);                    //delays for 10 cycles
  OSCCON = 0x72;                   // Switch to fast internal 8 MHz RC Clock

 // Set up port A
  LATA   = 0x00;                   // Pre-clear any outputs
  TRISA  = 0b00000000;             //set PortA as all outputs

  // Set up portB
  LATB   = 0x00;                   // Pre-clear any outputs
  TRISB  = 0b00010010;             //set SDA1(RB1) and SCK1(RB4) as inputs

  //initialize rxByte
  rxByte[0]=0;
    rxByte[1]=0;
      rxByte[2]=0;
        rxByte[3]=0;

  

  //Set up I2C control registers
   
   SSP1STATbits.SMP = 0 ;          //bit 7 = SMP
   SSP1STATbits.CKE = 1;           //bit 6 = CKE
   SSP1STATbits.BF = 0;            //initially clear the buffer

   //SSP1 Control register1
   SSP1CON1bits.SSPEN = 1;         //enable the serial port and configure the SDA1and
                                   //SCL1 pins
   SSP1CON1bits.CKP = 1;           //releases the clock
   SSP1CON1bits.SSPM= 0b0110;      //I2C Slave mode, 7-bit address
   
   //SSP1 Control register2
   SSP1CON2bits.GCEN = 0;          //General Call disabled
   SSP1CON2bits.SEN =  0;          /*Clock Stretch for both slave transmit
                                     and receive disabled*/

   //SSP1 Control register3
   SSP1CON3bits.BOEN = 0;         //SSP1BUF is only updated when SSP1OV is clear
   SSP1CON3bits.SDAHT =0;         // 100ns minimum hold time on the SDA1 line
   SSP1CON3bits.SBCDE =0;         //bus collision detection disabled
   SSP1CON3bits.AHEN = 0;         //address holding disabled
   SSP1CON3bits.DHEN = 0;         //data holding disabled

   //SSP1 MAsk register
   SSP1MSK= 0xFF;

   //Slave ID
   SSP1ADD = 0x04;  //Only the upper 7 bits are used for the address.Default value of 0x04


// clear interrupt flags
//  INTCONbits.TMR0IF=0;               // the timer interrupt flag
  PIR1bits.SSP1IF = 0;               // the i2c interrupt flag


//enable interrupts
  //INTCONbits.TMR0IE= 1;            //enable timer0 interrupt
  INTCONbits.PEIE= 1;                //enable peripheral interrupt
  PIE1bits.SSP1IE = 1;               //enable i2c interrupt
  ei();                              //enable global interrupts

  while (1){
       LATAbits.LATA6=1;
  if (i2c_complete==1){
 
  if (rxByte[1]==0x01){  //command #01
                //light up port A2
                    LATAbits.LATA2=1;
                    LATAbits.LATA3=0;
                    LATAbits.LATA4=0;
                }
  if (rxByte[1]==0x02){  //command #01
                //light up port A2
                    LATAbits.LATA3=1;
                    LATAbits.LATA2=0;
                    LATAbits.LATA4=0;
                }
  if (rxByte[1]==0x03){  //command #01
                //light up port A2
                    LATAbits.LATA4=1;
                    LATAbits.LATA3=0;
                    LATAbits.LATA2=0;
                }
  i2c_complete=0;
  }


  }

}
