/******************************************************************************
 *  Filename:       timer.c
 *  Date:           03/15/13
 *  File Verison:   1.1
 *
 *  Author:         Amira Ahsan
 *  Team:           LFEV'13
 *
 * ****************************************************************************
 *  Architecture:   Enhanced Midrange PIC
 *  Processor:      PIC16LF1827
 *  Compiler:       MPLAB X XC8
 *
 *
 * ****************************************************************************
 *  Description:    Uses a Timer0 interrupt to flash a LED at
 *                  0.5 Hz( 50% duty cycle)
 *
 * ****************************************************************************
 *  Pin Assignments:
 *                  RA6= green LED
 *                  RA1= red LED
 *
 *******************************************************************************/

#include <xc.h>
#include <math.h>
#include <stdint.h>


// configuring the bits
#pragma config MCLRE= OFF        //MCLR/VPP pin function is digital input
#pragma config FOSC = INTOSC     // Selects the internal oscillator
#pragma config WDTE = OFF        //Turns off the Watchdog Timer
#pragma config LVP = OFF         //Low-voltage programming disabled
#pragma config PLLEN= OFF

#define TOFFSET (256-250+3)      //timer0 offset such that it overflows
                                 //after 250 cycles - 125 microsecond


//the interupt service routine
void interrupt isr(void){
    static uint16_t cnt_t0=0;    // to count number of overflows
    
    TMR0+=TOFFSET;             //offset the Timer0 register
    INTCONbits.TMR0IF=0;       //clear the interrupt flag

    cnt_t0++;                  //increase the counter
   

    if(cnt_t0 == 8000){                          //cnt_0= (1s/125u)
        cnt_t0=0;                               //reset  the counter
        LATAbits.LATA6 = ~(LATAbits.LATA6);     //toogle the LED
     }
     
}




void main(){
              
  _delay(1000);                        //delays for 10 cycles
  OSCCON = 0x72;                       // Switch to fast internal 8 MHz RC Clock

 // Set up port A
  LATA   = 0x00;                    // Pre-clear any outputs
  TRISA  = 0b00000000;              //set PortA as all outputs


 //initially light up the led
  LATAbits.LATA6= 1;

 //configure timer0
  OPTION_REGbits.TMR0CS = 0;        //Timer0 clock source- internal instruction cycle clock (FOSC/4)
  OPTION_REGbits.PSA = 1;           //Timer0 Prescale assignment bit- no prescale assigned

// set up the interrupts

// clear the flag first
  INTCONbits.TMR0IF=0;

//enable interrupts
  INTCONbits.TMR0IE= 1;              //enable timer0 interrupt
  ei();                              //enable global interrupts




  while(1){
  //LATAbits.LATA1 = 1;               //LED to signal power

  }


}
